package apitester

import (
	"net/http"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/auth"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// Register registers the user to the API.
func (tester *APITester) Register(reg *models.Registration) {
	apitest := tester.APITest("register")
	if tester.debug {
		apitest = apitest.Debug()
	}

	_ = apitest.
		Post("/api/v1/auth/register").
		JSON(reg).
		Expect(tester.t).
		Status(http.StatusNoContent).
		End()
}

// ForgotPassword sends an email to the user to reset his password.
func (tester *APITester) ForgotPassword(body *auth.ForgotPasswordBody) {
	apitest := tester.APITest("forgot_password")
	if tester.debug {
		apitest = apitest.Debug()
	}

	_ = apitest.
		Post("/api/v1/auth/forgot-password").
		JSON(body).
		Expect(tester.t).
		Status(http.StatusNoContent).
		End()
}

// ResetPassword reset the password.
func (tester *APITester) ResetPassword(body *auth.ResetPasswordBody) {
	apitest := tester.APITest("reset_password")
	if tester.debug {
		apitest = apitest.Debug()
	}

	_ = apitest.
		Post("/api/v1/auth/reset-password").
		JSON(body).
		Expect(tester.t).
		Status(http.StatusNoContent).
		End()
}

// SetPassword sets the user's password.
func (tester *APITester) SetPassword(username, password string) {
	apitest := tester.APITest("password_set")
	if tester.debug {
		apitest = apitest.Debug()
	}

	body := user_management.UserPasswordSetBody{Password: password}
	_ = apitest.
		Post("/api/v1/user/" + username + "/password").
		JSON(body).
		Expect(tester.t).
		Status(http.StatusNoContent).
		End()
}

// Login authenticates the user for the subsequent requests.
func (tester *APITester) Login(username, password string) {
	apitest := tester.APITest("login")
	if tester.debug {
		apitest = apitest.Debug()
	}

	creds := models.Credentials{
		Username: username,
		Password: password,
	}

	var token string

	r := apitest.
		Post("/api/v1/auth/authenticate").
		JSON(&creds).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	r.JSON(&token)
	tester.SetHeader("Authorization", "Bearer "+token)
}

// Logout resets the "Authorization" headers.
func (tester *APITester) Logout() {
	tester.SetHeader("Authorization", "")
}

// GetUserInfo returns the connected user information.
func (tester *APITester) GetUserInfo() models.Userinfo {
	apitest := tester.APITest("login")
	if tester.debug {
		apitest = apitest.Debug()
	}

	var user models.Userinfo

	r := apitest.
		Get("/api/v1/auth/userinfo").
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	r.JSON(&user)

	return user
}
